@@ -1,7 +1,9 @@
 use std::collections::BTreeMap;
 use std::collections::HashMap;
+
 use std::fs::File;
 use std::io::{self, BufRead, BufReader};
+use std::process::Command;
 use std::sync::Arc;
 
 use regex::Regex;
@@ -45,7 +47,7 @@ pub struct Info {
     others: String,
 }
 
-pub fn extract_info(rawline: &str, config: &Config) -> Option<HashMap<String, String>> {
+pub fn extract_info(rawline: &str, config: &Config) -> HashMap<String, String> {
     let time_format = config.time_format.as_str();
     let mut msg = HashMap::new();
 
@@ -64,31 +66,13 @@ pub fn extract_info(rawline: &str, config: &Config) -> Option<HashMap<String, St
             .replace("{namespace}", namespace)
             .replace("{pod}", pod)
             .replace("{container}", container);
+    } else {
+        kali_msg_prefix = String::new();
     }
 
     if !config.json_keys.is_empty() {
-        if let Ok(p) = serde_json::from_str::<Generic>(line.as_str()) {
-            for (key, value) in p.other {
-                if config.json_keys.contains_key(key.as_str()) {
-                    if config.json_keys[key.as_str()].as_str() == "ts" {
-                        msg.insert(
-                            String::from("ts"),
-                            crate::utils::conver_ts_float_or_str(&value, time_format),
-                        );
-                    } else {
-                        msg.insert(
-                            config.json_keys[key.as_str()].clone(),
-                            value.as_str().unwrap().to_string(),
-                        );
-                    }
-                }
-            }
-            if !config.kail_no_prefix && !kali_msg_prefix.is_empty() && msg.contains_key("msg") {
-                *msg.get_mut("msg").unwrap() =
-                    format!("{} {}", Paint::blue(kali_msg_prefix), msg["msg"])
-            }
-            return Some(msg);
-        }
+        msg = custom_json_match(config, time_format, kali_msg_prefix.as_str(), line.as_str())
+            .unwrap();
     }
 
     if let Ok(p) = serde_json::from_str::<Pac>(line.as_str()) {
@@ -127,60 +111,119 @@ pub fn extract_info(rawline: &str, config: &Config) -> Option<HashMap<String, St
     }
 
     if !config.kail_no_prefix && !kali_msg_prefix.is_empty() && msg.contains_key("msg") {
-        *msg.get_mut("msg").unwrap() = format!("{} {}", Paint::blue(kali_msg_prefix), msg["msg"])
+        *msg.get_mut("msg").unwrap() = format!("{} {}", Paint::blue(kali_msg_prefix), msg["msg"]);
     }
-    Some(msg)
+    msg
 }
 
-pub(crate) fn parse_line(config: Arc<Config>, line: &str) -> Option<Info> {
+fn custom_json_match(
+    config: &Config,
+    time_format: &str,
+    kali_msg_prefix: &str,
+    line: &str,
+) -> Option<HashMap<String, String>> {
+    let mut dico = HashMap::new();
+    if let Ok(p) = serde_json::from_str::<Generic>(line) {
+        for (key, value) in p.other {
+            if config.json_keys.contains_key(key.as_str()) {
+                match config.json_keys[key.as_str()].as_str() {
+                    "ts" => {
+                        dico.insert(
+                            String::from("ts"),
+                            crate::utils::conver_ts_float_or_str(&value, time_format),
+                        );
+                    }
+                    _ => {
+                        dico.insert(
+                            config.json_keys[key.as_str()].clone(),
+                            value.as_str().unwrap().to_string(),
+                        );
+                    }
+                }
+            }
+        }
+        if !config.kail_no_prefix && !kali_msg_prefix.is_empty() && dico.contains_key("msg") {
+            *dico.get_mut("msg").unwrap() =
+                format!("{} {}", Paint::blue(kali_msg_prefix), dico["msg"]);
+        }
+        return Some(dico);
+    }
+    None
+}
+
+pub fn action_on_regexp(config: &Config, line: &str) {
+    if config.action_regexp.is_empty() || config.action_command.is_empty() {
+        return;
+    }
+    let action_regexp = Regex::new(config.action_regexp.as_str()).unwrap();
+    if let Some(reg) = action_regexp.captures(line) {
+        let regexpmatch = reg.get(0).unwrap().as_str();
+        // replace {} by the actual match
+        let action_command = config.action_command.replace("{}", regexpmatch);
+        if Command::new("sh")
+            .arg("-c")
+            .arg(action_command)
+            .spawn()
+            .is_ok()
+        {
+            println!(
+                "Spawned command: {} for action: {}",
+                Paint::yellow(&config.action_command),
+                Paint::cyan(action_regexp)
+            );
+        }
+    }
+}
+
+fn do_line(config: &Config, line: &str) -> Option<Info> {
     // exclude lines with only space or empty
     if line.trim().is_empty() {
         return None;
     }
 
-    if let Some(msg) = extract_info(line, &config) {
-        let unwrapped = serde_json::to_string(&msg).unwrap();
-        //check if unwrapped is not an empty hashmap
-        if unwrapped == "{}" {
-            println!(
-                "{}",
-                apply_regexps(&config.regexp_colours, line.to_string())
-            );
-            return None;
-        }
-        if !config.filter_levels.is_empty()
-            && !config.filter_levels.contains(&msg["level"].to_lowercase())
-        {
-            return None;
-        }
+    action_on_regexp(config, line);
 
-        let mut level = crate::utils::color_by_level(msg.get("level").unwrap());
-        if config.level_symbols {
-            level = crate::utils::level_symbols(msg.get("level").unwrap());
-        }
-        let mut ts = String::new();
-        if msg.contains_key("ts") {
-            ts = Paint::fixed(13, msg.get("ts").unwrap()).to_string();
-        }
-        let other = if msg.contains_key("others") {
-            format!(" {}", Paint::cyan(msg.get("others").unwrap()).italic())
-        } else {
-            "".to_string()
-        };
-        let mut themsg = msg.get("msg").unwrap().to_string();
-
-        if !config.regexp_colours.is_empty() {
-            themsg = apply_regexps(&config.regexp_colours, themsg)
-        }
-        Some(Info {
-            level,
-            timestamp: ts,
-            others: other,
-            msg: themsg,
-        })
-    } else {
-        None
+    let msg = extract_info(line, config);
+    let unwrapped = serde_json::to_string(&msg).unwrap();
+    //check if unwrapped is not an empty hashmap
+    if unwrapped == "{}" {
+        println!(
+            "{}",
+            apply_regexps(&config.regexp_colours, line.to_string())
+        );
+        return None;
     }
+
+    if !config.filter_levels.is_empty()
+        && !config.filter_levels.contains(&msg["level"].to_lowercase())
+    {
+        return None;
+    }
+
+    let mut level = crate::utils::color_by_level(msg.get("level").unwrap());
+    if config.level_symbols {
+        level = crate::utils::level_symbols(msg.get("level").unwrap());
+    }
+    let mut ts = String::new();
+    if msg.contains_key("ts") {
+        ts = Paint::fixed(13, msg.get("ts").unwrap()).to_string();
+    }
+    let other = if msg.contains_key("others") {
+        format!(" {}", Paint::cyan(msg.get("others").unwrap()).italic())
+    } else {
+        "".to_string()
+    };
+    let mut themsg = msg.get("msg").unwrap().to_string();
+
+    if !config.regexp_colours.is_empty() {
+        themsg = apply_regexps(&config.regexp_colours, themsg);
+    }
+    Some(Info {
+        level,
+        timestamp: ts,
+        others: other,
+        msg: themsg,
+    })
 }
 
 pub fn apply_regexps(regexps: &HashMap<String, Color>, msg: String) -> String {
@@ -195,12 +238,12 @@ pub fn apply_regexps(regexps: &HashMap<String, Color>, msg: String) -> String {
     ret
 }
 
-pub fn read_from_stdin(config: Arc<Config>) {
+pub fn read_from_stdin(config: &Arc<Config>) {
     let stdin = io::stdin();
     for line in stdin.lock().lines() {
         let parseline = &line.unwrap();
 
-        if let Some(info) = parse_line(config.clone(), parseline) {
+        if let Some(info) = do_line(config, parseline) {
             println!(
                 "{} {} {}{}",
                 info.level, info.timestamp, info.others, info.msg
@@ -209,23 +252,34 @@ pub fn read_from_stdin(config: Arc<Config>) {
     }
 }
 
-pub fn read_from_files(config: Arc<Config>) {
-    for f in &config.files {
-        // open file and parse each lines
-        let file = File::open(f).map_err(|e| {
-            eprintln!("file {}, {}", f, e);
-            std::process::exit(1);
-        });
-        let buf_reader = BufReader::new(file.unwrap());
-        for line in buf_reader.lines() {
-            let parseline = &line.unwrap();
+// read from a bunch files and pass read_from_stdin to stdout
+pub fn read_from_files(config: &Arc<Config>) {
+    for filename in &config.files {
+        // write to a BufWriter that output to stdout
+        let stdout = io::stdout();
+        let stdout = stdout.lock();
+        let mut stdout = io::BufWriter::new(stdout);
+        read_from_file(config, filename, &mut stdout);
+    }
+}
 
-            if let Some(info) = parse_line(config.clone(), parseline) {
-                println!(
-                    "{} {} {}{}",
-                    info.level, info.timestamp, info.others, info.msg
-                );
-            }
+// read from file and output to the writer. This makes it easy to unittest
+pub fn read_from_file(config: &Config, filename: &str, writeto: &mut dyn io::Write) {
+    let file = File::open(filename).map_err(|e| {
+        eprintln!("file {}, {}", filename, e);
+        std::process::exit(1);
+    });
+    let buf_reader = BufReader::new(file.unwrap());
+    for line in buf_reader.lines() {
+        let parseline = &line.unwrap();
+
+        if let Some(info) = do_line(config, parseline) {
+            writeln!(
+                writeto,
+                "{} {} {}{}",
+                info.level, info.timestamp, info.others, info.msg
+            )
+            .unwrap();
         }
     }
 }
